#include "common.fx"

SamplerState SampleType;

#define DIFFUSE_MAP 0
#define NORMAL_MAP 1

Texture2D shaderTextures[2] : register( t1 );

cbuffer ObjectBufferCustom : register(b5) 
{
	// X, Y = Blur direction
	// Z = Sigma	
	float2 myBlurDirection;
	float mySigma;
	float padding1;
};

static const float2 targetResolution = float2(320.0f, 180.0f);

float gaussian_formula(float aPos)
{
	return exp(-(aPos * aPos) / (2.f * mySigma * mySigma));
}

float4 PShader(PixelInputType input) : SV_TARGET
{
	const float blurAmount = 5.f;

	const float kernel = 2.f * blurAmount + 1.f;
	float totalWeight = 1.f;

	const float texelSize = 1.0f / targetResolution.x;

	float4 blurredColor = shaderTextures[DIFFUSE_MAP].Sample(SampleType, input.tex);
	
	// NOTE: Having the float in the for loop seems to crash the driver?
	for (int i = 1; i < blurAmount; ++i)
	{
		const float offset = (float)i;
	
		float weight = gaussian_formula(offset / kernel);
		
		totalWeight += 2.f * weight;
		
		float2 leftPixel = input.tex.xy - offset * texelSize * myBlurDirection;
		blurredColor += shaderTextures[DIFFUSE_MAP].Sample(SampleType, leftPixel) * weight;
	
		float2 rightPixel = input.tex.xy + offset * texelSize * myBlurDirection;
		blurredColor += shaderTextures[DIFFUSE_MAP].Sample(SampleType, rightPixel) * weight;
	}

	return blurredColor / totalWeight;
}